/**
 * @file Manages the secure HTTP connection
 */

package vkey.android.cordova;

import android.content.Context;
import android.text.TextUtils;
import android.util.Log;

import com.vkey.android.secure.net.Response;
import com.vkey.android.secure.net.SecureHttpUrlConnection;
import com.vkey.android.vguard.VGException;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;
import java.util.Map;

public class SecureHTTPConnectionPlugin {
    private static SecureHTTPConnectionPlugin sInstance;

    private static final String RET_CODE_SUCCESS = "1";
    private static final String CODE_VOS_NOT_STARTED = "-1000";
    private static final String CODE_INVALID_PARAMS = "-1001";

    private static final String ACT_MTLS_GET = "act_mtls_get";
    private static final String ACT_MTLS_POST = "act_mtls_post";

    @SuppressWarnings("unused")
    private static final String TAG = SecureHTTPConnectionPlugin.class.getName();

    private CordovaPlugin mPlugin;
    private SecureHTTPConnectionPlugin(CordovaPlugin plugin) {
        mPlugin = plugin;
    }

    private boolean checkVosStatus(CallbackContext callbackContext) {
//        boolean isVosStarted = VGuardSetupPlugin.getInstance(mPlugin).getIsVosStarted();
        boolean isVosStarted = true;
        // try {
        //     isVosStarted = (new VGuardFactory()).getVGuard(mPlugin.cordova.getActivity()).getIsVosStarted();
        //     if(!isVosStarted) {
        //         Log.d("SecureHTTP", "V-OS not started");
        //         PluginResult result = new PluginResult(PluginResult.Status.ERROR, CODE_VOS_NOT_STARTED);
        //         result.setKeepCallback(true);
        //         callbackContext.sendPluginResult(result);
        //     }
        // } catch (Exception e) {
        //     e.printStackTrace();
        // }

        return isVosStarted;
    }
    private void handleRespone(Response r, CallbackContext callbackContext){
        if(r != null){
            int responseCode = r.responseCode;
            Log.d("SecureHTTP", "return code" + responseCode);
            Map headers = r.headers;
            if(responseCode >= 200 && responseCode < 300){
                try {
                    callbackContext.success((new JSONObject())
                            .put("responseCode",r.responseCode)
                            .put("response",byteToJSonObject(r.response))
                            .put("header", headers)
                    );
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }else if(responseCode >= 400){
                callbackContext.error(responseCode);
            }
        }
    }

    public boolean execute(String action, final List<String> args, final CallbackContext callbackContext){
        //Check Null params
        if(args.size() == 0){
            callbackContext.error(CODE_INVALID_PARAMS);
            return false;
        }
        if(action.equals(ACT_MTLS_GET)){
            if(!checkVosStatus(callbackContext)){
                callbackContext.error(CODE_VOS_NOT_STARTED);
            }
            //CHECK NULL
            //Check hostname
            if (checkStringNull(getArgAtIndex(args,0), callbackContext)) {
                return false;
            }
            //check endpt
            if (checkStringNull(getArgAtIndex(args,1), callbackContext)) {
                return false;
            }
            //check timeout
            if (checkIntegerNull(getArgAtIndex(args,3), callbackContext)) {
                return false;
            }
            //END
            threadHelper((new secureHTTP() {
                @Override
                public void exec() {
                    Context ctx = mPlugin.cordova.getActivity().getApplicationContext();
                    try {
                        Response r = SecureHttpUrlConnection.get_urlconnection(
                            getArgAtIndex(args,0),
                            getArgAtIndex(args,1),
                            ctx,
                            getArgAtIndex(args,2),
                            Integer.parseInt(getArgAtIndex(args,3))
                        );
                        handleRespone(r, callbackContext);
                    } catch (VGException e) {
                        e.printStackTrace();
                    }
                }
            }));
        }else if(action.equals(ACT_MTLS_POST)){
            if(!checkVosStatus(callbackContext)){
                callbackContext.error(CODE_VOS_NOT_STARTED);
            }
            //CHECK NULL
            //Check hostname
            if (checkStringNull(getArgAtIndex(args,0), callbackContext)) {
                return false;
            }
            //check endpt
            if (checkStringNull(getArgAtIndex(args,1), callbackContext)) {
                return false;
            }
            //check content Type
            if (checkStringNull(getArgAtIndex(args,2), callbackContext)) {
                return false;
            }
            // check data
            if (checkStringNull(getArgAtIndex(args,3), callbackContext)) {
                return false;
            }
            //check timeout
            if (checkIntegerNull(getArgAtIndex(args,5), callbackContext)) {
                return false;
            }
            //END
            threadHelper((new secureHTTP() {
                @Override
                public void exec() {
                    Context ctx = mPlugin.cordova.getActivity().getApplicationContext();
                    try {
                        Response r = SecureHttpUrlConnection.post_urlconnection(
                            getArgAtIndex(args,0),
                            getArgAtIndex(args,1),
                            getArgAtIndex(args,2),
                            getArgAtIndex(args,3),
                            ctx,
                            getArgAtIndex(args,4),
                            Integer.parseInt(getArgAtIndex(args,5))
                        );
                        handleRespone(r, callbackContext);
                    } catch (VGException e) {
                        e.printStackTrace();
                    }
                }
            }));
        }else{
            return false;
        }
        return true;
    }
    private interface secureHTTP{
        void exec();
    }
    private void threadHelper(final secureHTTP s){
        mPlugin.cordova.getThreadPool().execute(new Runnable() {
            public void run() {
                try {
                    s.exec();
                } catch ( Exception e) {
                    Log.d("SecureHTTP", e.getMessage());
                }
            }
        });
    }

    private JSONObject byteToJSonObject(byte[] response){
        if (response != null){
            try {
                return new JSONObject(new String(response));
            } catch (JSONException e) {
                e.printStackTrace();
                return null;
            }
        }
        return null;
    }

    public static SecureHTTPConnectionPlugin getInstance(CordovaPlugin plugin) {
        if(sInstance == null || sInstance.mPlugin == null) {
            sInstance = new SecureHTTPConnectionPlugin(plugin);
        }
        return sInstance;
    }
    private static String getArgAtIndex(List<String> args, int index) {
        String value = "";
        try {
            value = args.get(index);
            value = value.equals("null") ? null : value;
        }
        catch(IndexOutOfBoundsException ioe) {
            ioe.printStackTrace();
        }
        return value;
    }

    private boolean checkStringNull(String value, CallbackContext callbackContext) {
        if(TextUtils.isEmpty(value)) {
            callbackContext.error(CODE_INVALID_PARAMS);
            return true;
        }
        return false;
    }

    private boolean checkIntegerNull(String value,  CallbackContext callbackContext) {
        if(checkStringNull(value, callbackContext)) {
            return true;
        }
        try {
            int intValue = Integer.parseInt(value);
        } catch ( Exception e) {
            callbackContext.error(CODE_INVALID_PARAMS);
            return true;
        }
        return false;
    }
}