/*
 * (C) Copyright 2016 V-Key Solutions.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 * 		Rukmal Dias
 */

package vkey.android.cordova;

import java.util.LinkedList;
import java.util.List;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONException;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.util.Log;
import vkey.android.cordova.preference.PreferencePlugin;
import vkey.android.cordova.SecureHTTPConnectionPlugin;

public class VGuardPlugin extends CordovaPlugin {
	
	//-- VGuard plug-in actions
	private static final String ACT_ALERT_TEST = "act_alert_test";
	
	// VGuard setup
	private static final String ACT_SETUP_VGUARD = "act_setup_vguard";
	private static final String ACT_SEND_EMAIL = "act_send_email";
	private static final String ACT_VGUARD_PAUSE = "act_vguard_pause";
	private static final String ACT_VGUARD_RESUME = "act_vguard_resume";
	
	// Secure Shared Preferences
	private static final String ACT_SP_PUT = "act_sp_put";
	private static final String	ACT_SP_GET = "act_sp_get";
	private static final String ACT_SP_REMOVE = "act_sp_remove";
	private static final String ACT_SP_CONTAINS_KEY = "act_sp_contains_key";
	private static final String ACT_SP_CLEAR = "act_sp_clear";
	
	// SQLite
	private static final String ACT_ECHO_STRING = "echoStringValue";
	private static final String ACT_OPEN_DB = "open";
	private static final String ACT_CLOSE_DB = "close";
	private static final String ACT_DELETE_DB = "delete";
	private static final String ACT_EXEC_SQL_BATCH = "executeSqlBatch";
	private static final String ACT_BG_EXEC_SQL_BATCH = "backgroundExecuteSqlBatch";
	
	// Secure file IO
	private PreferencePlugin mPrefPlugin;
	private VGuardSetupPlugin mVgPlugin;
//	private SQLitePlugin mSQLitePlugin;

	//mTLS
	private SecureHTTPConnectionPlugin mtlsPlugin;
	private static final String ACT_MTLS_GET = "act_mtls_get";
	private static final String ACT_MTLS_POST = "act_mtls_post";
	
	private static final String TAG = VGuardPlugin.class.getSimpleName();
	

	@Override
	protected void pluginInitialize() {
		mPrefPlugin = PreferencePlugin.getInstance(this);
		mVgPlugin = VGuardSetupPlugin.getInstance(this);
//		mSQLitePlugin = SQLitePlugin.getInstance(this);
		mtlsPlugin = SecureHTTPConnectionPlugin.getInstance(this);
		// VGuardSetupPlugin.getInstance(this).setupVGuardFromNative();
		//DigbankVN
		// mKeyboardPlugin = KeyboardPlugin.getInstance(this); 

	}
	
	@Override
	public boolean execute(String action, final String rawArgs, final CallbackContext callbackContext) {
		// if second parameter is JSONArray go to second execute method
		JSONArray jsonArray = null;
		try {
			jsonArray = new JSONArray(rawArgs);
		} 
		catch (JSONException e) {
			Log.d(TAG, "-- execute " + e.getMessage());
		}
		
		if (action == null) {
			callbackContext.sendPluginResult(new PluginResult(PluginResult.Status.ERROR, "Action can not be null"));
			return false;
		}
		else if (action.equals(ACT_ALERT_TEST)) {
			showTestAlert(rawArgs, callbackContext);
		}
		else if(action.equals(ACT_SETUP_VGUARD)) {
			// 1. Call set up VGuard
			mVgPlugin.setupVGuard(callbackContext);
		}
		else if(action.equals(ACT_VGUARD_PAUSE)) {
			mVgPlugin.onPause(callbackContext);
		}
		else if(action.equals(ACT_VGUARD_RESUME)) {
			mVgPlugin.onResume(callbackContext);
		}
		else if(action.equals(ACT_SEND_EMAIL)) {
			Log.d("send email","send email in Vguard Plugin" );
            List<String> args = getArgsList(rawArgs);
            String email = getArgAtIndex(args, 0);
			mVgPlugin.sendEmail(email, callbackContext);
		}
		else if(action.equals(ACT_SP_PUT)) {
			List<String> args = getArgsList(rawArgs);
			String key = getArgAtIndex(args, 0);
			String value = getArgAtIndex(args, 1);
			mPrefPlugin.spput(callbackContext, key, value);
		}
		else if(action.equals(ACT_SP_GET)) {
			List<String> args = getArgsList(rawArgs);
			String key = getArgAtIndex(args, 0);
			mPrefPlugin.spget(callbackContext, key);
		}
		else if(action.equals(ACT_SP_REMOVE)) {
			List<String> args = getArgsList(rawArgs);
			String key = getArgAtIndex(args, 0);
			mPrefPlugin.spremove(callbackContext, key);
		}
		else if(action.equals(ACT_SP_CONTAINS_KEY)) {
			List<String> args = getArgsList(rawArgs);
			String key = getArgAtIndex(args, 0);
			mPrefPlugin.spcontainskey(callbackContext, key);
		}
		else if(action.equals(ACT_SP_CLEAR)) {
			mPrefPlugin.spclear(callbackContext);
		}
//		else if (action.equals(ACT_ECHO_STRING)
//				|| action.equals(ACT_OPEN_DB)
//				|| action.equals(ACT_CLOSE_DB)
//				|| action.equals(ACT_DELETE_DB)
//			    || action.equals(ACT_EXEC_SQL_BATCH)
//			    || action.equals(ACT_BG_EXEC_SQL_BATCH)) {
//			mSQLitePlugin.execute(action, jsonArray, callbackContext);
//		}
		// DigbankVN
		else if(action.equals(ACT_MTLS_GET)
				||action.equals(ACT_MTLS_POST)){
			List<String> args = getArgsList(rawArgs);
			mtlsPlugin.execute(action, args, callbackContext);
		
		}else {
			return false;
		}
		return true;
	}
	
//	@Override
//	public boolean execute(String action, JSONArray args, CallbackContext callbackContext) {
//		if (action == null) {
//			callbackContext.sendPluginResult(new PluginResult(PluginResult.Status.ERROR, "Action can not be null"));
//			return true;
//		}
//		else if (action.equals(ACT_ECHO_STRING)
//				|| action.equals(ACT_OPEN_DB)
//				|| action.equals(ACT_CLOSE_DB)
//				|| action.equals(ACT_DELETE_DB)
//			    || action.equals(ACT_EXEC_SQL_BATCH)
//			    || action.equals(ACT_BG_EXEC_SQL_BATCH)) {
//			mSQLitePlugin.execute(action, args, callbackContext);
//		}
//		else {
//			return false;
//		}
//		return true;
//	}
	
	@Override
	public void onDestroy() {
//		if(mSQLitePlugin != null) {
//			mSQLitePlugin.onDestroy();
//		}
	}
	
	/**
	 * Shows a test alert
	 * @param rawArgs
	 * @param callbackContext
	 */
	public synchronized void showTestAlert(final String rawArgs, final CallbackContext callbackContext) {
		String title ="", message = "", buttonLabel = "";
		JSONArray args;
		try {
			args = new JSONArray(rawArgs);
			title = args.getString(0);
			message = args.getString(1);
			buttonLabel = args.getString(2);
		} 
		catch (JSONException e) {
		}
		
		new AlertDialog.Builder(cordova.getActivity())
		  	.setTitle(title)
		  	.setMessage(message)
		  	.setCancelable(false)
		  	.setNeutralButton(buttonLabel, new AlertDialog.OnClickListener() {
		  		public void onClick(DialogInterface dialogInterface, int which) {
		  			dialogInterface.dismiss();
		  			callbackContext.sendPluginResult(new PluginResult(PluginResult.Status.OK, 0));
		  		}
		  	})
		  	.create()
		  	.show();
	}
	
	private List<String> getArgsList(String rawArgs) {
		List<String> list = new LinkedList<String>();
		JSONArray args;
		try {
			args = new JSONArray(rawArgs);
			if(args != null && args.length()>0) {
				for (int i=0; i<args.length(); i++) {
					list.add(args.getString(i));
				}
			}
		} 
		catch (JSONException e) {
			Log.d(TAG, "-- getArgsList" + e.getMessage());
		}
		return list;
	}
	
	private String getArgAtIndex(List<String> args, int index) {
		String value = "";
		try {
			value = args.get(index);
		}
		catch(IndexOutOfBoundsException ioe) {
		}
		return value;
	}

}
