/*
 * (C) Copyright 2016 V-Key Solutions.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 * 		Rukmal Dias
 */

package vkey.android.cordova.keyboard;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.PluginResult;

import com.vkey.android.internal.vguard.secure.preference.SecurePreferences;

import android.content.Context;
import vkey.android.cordova.VGuardSetupPlugin;

public class KeyboardPlugin {

	private static KeyboardPlugin sInstance;
	
	private static final String RET_CODE_SUCCESS = "1";
	private static final String RET_MESSAGE_VGUARD_NOT_STARTED = "vguard is not started..";
	
	@SuppressWarnings("unused")
	private static final String TAG = KeyboardPlugin.class.getName();
	
	private CordovaPlugin mPlugin;
	
	private KeyboardPlugin(CordovaPlugin plugin) {
		mPlugin = plugin;
	}
	
	/**
	 * Saves key, value in secure preferences
	 * @param callbackContext
	 * @param key
	 * @param value
	 */
    public void spput(CallbackContext callbackContext, String key, String value) {
    	// check whether vos is started or not..
    	boolean isVosStarted = VGuardSetupPlugin.getInstance(mPlugin).getIsVosStarted();
    	if(isVosStarted) {
    		Context context = mPlugin.cordova.getActivity().getApplicationContext(); 
    		SecurePreferences sp = SecurePreferences.getInstance(context);
    		try {
    			sp.put(key, value);
    			PluginResult result = new PluginResult(PluginResult.Status.OK, RET_CODE_SUCCESS);
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    		catch(Exception e) {
    			PluginResult result = new PluginResult(PluginResult.Status.ERROR, e.getMessage());
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    	}
    	else {
			PluginResult result = new PluginResult(PluginResult.Status.ERROR, RET_MESSAGE_VGUARD_NOT_STARTED);
	    	result.setKeepCallback(true);
	    	callbackContext.sendPluginResult(result);
    	}
    }
    
    /**
     * Returns a stored value from secure preferences
     * @param callbackContext
     * @param key
     */
    public void spget(CallbackContext callbackContext, String key) {
    	// check whether vos is started or not..
    	boolean isVosStarted = VGuardSetupPlugin.getInstance(mPlugin).getIsVosStarted();
    	String value = null;
    	if(isVosStarted) {
    		Context context = mPlugin.cordova.getActivity().getApplicationContext(); 
    		SecurePreferences sp = SecurePreferences.getInstance(context);
    		try {
    			value = sp.get(key);
    			PluginResult result = new PluginResult(PluginResult.Status.OK, value);
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    		catch(Exception e) {
    			PluginResult result = new PluginResult(PluginResult.Status.ERROR, e.getMessage());
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    	}
    	else {
    		PluginResult result = new PluginResult(PluginResult.Status.ERROR, RET_MESSAGE_VGUARD_NOT_STARTED);
	    	result.setKeepCallback(true);
	    	callbackContext.sendPluginResult(result);
    	}
    }
    
    /**
     * Removes a key, value pair from secure preferences
     * @param callbackContext
     * @param key
     */
    public void spremove(CallbackContext callbackContext, String key) {
    	// check whether vos is started or not..
    	boolean isVosStarted = VGuardSetupPlugin.getInstance(mPlugin).getIsVosStarted();
    	if(isVosStarted) {
    		Context context = mPlugin.cordova.getActivity().getApplicationContext(); 
    		SecurePreferences sp = SecurePreferences.getInstance(context);
    		try {
    			sp.remove(key);
    			PluginResult result = new PluginResult(PluginResult.Status.OK, RET_CODE_SUCCESS);
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    		catch(Exception e) {
    			PluginResult result = new PluginResult(PluginResult.Status.ERROR, e.getMessage());
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    	}
    	else {
    		PluginResult result = new PluginResult(PluginResult.Status.ERROR, RET_MESSAGE_VGUARD_NOT_STARTED);
	    	result.setKeepCallback(true);
	    	callbackContext.sendPluginResult(result);
    	}
    }
    
    /**
     * Finds whether a particular key exists in secure preferences
     * @param callbackContext
     * @param key
     */
    public void spcontainskey(CallbackContext callbackContext, String key) {
    	// check whether vos is started or not..
    	boolean isVosStarted = VGuardSetupPlugin.getInstance(mPlugin).getIsVosStarted();
    	String value = null;
    	if(isVosStarted) {
    		Context context = mPlugin.cordova.getActivity().getApplicationContext(); 
    		SecurePreferences sp = SecurePreferences.getInstance(context);
    		try {
    			boolean contains = sp.containsKey(key);
    			value = ((contains)? "1" : "0" );
    			PluginResult result = new PluginResult(PluginResult.Status.OK, value);
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    		catch(Exception e) {
    			PluginResult result = new PluginResult(PluginResult.Status.ERROR, e.getMessage());
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    	}
    	else {
    		PluginResult result = new PluginResult(PluginResult.Status.ERROR, RET_MESSAGE_VGUARD_NOT_STARTED);
	    	result.setKeepCallback(true);
	    	callbackContext.sendPluginResult(result);
    	}
    }
    
    /**
     * Removes all key, value pairs from secure preferences
     * @param callbackContext
     */
    public void spclear(CallbackContext callbackContext) {
    	// check whether vos is started or not..
    	boolean isVosStarted = VGuardSetupPlugin.getInstance(mPlugin).getIsVosStarted();
    	if(isVosStarted) {
    		Context context = mPlugin.cordova.getActivity().getApplicationContext(); 
    		SecurePreferences sp = SecurePreferences.getInstance(context);
    		try {
    			sp.clear();
    			PluginResult result = new PluginResult(PluginResult.Status.OK, RET_CODE_SUCCESS);
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    		catch(Exception e) {
    			PluginResult result = new PluginResult(PluginResult.Status.ERROR, e.getMessage());
    	    	result.setKeepCallback(true);
    	    	callbackContext.sendPluginResult(result);
    		}
    	}
    	else {
    		PluginResult result = new PluginResult(PluginResult.Status.ERROR, RET_MESSAGE_VGUARD_NOT_STARTED);
	    	result.setKeepCallback(true);
	    	callbackContext.sendPluginResult(result);
    	}
    }
    
	public static KeyboardPlugin getInstance(CordovaPlugin plugin) {
		if(sInstance == null || sInstance.mPlugin == null) {
			sInstance = new KeyboardPlugin(plugin);
		}
		return sInstance;
	}

}