//
//  SecureHTTPConnectionPlugin.m
//
//  Created by Nguyen Thuy An (An) on 7/5/19.
//

#import "SecureHTTPConnectionPlugin.h"
#import <VGuard/VGuard.h>

#pragma mark -

@interface SecureHTTPConnectionPlugin () {
}
@end

#pragma mark -

@implementation SecureHTTPConnectionPlugin

static NSString* CODE_INVALID_PARAMS = @"-1001";

-(BOOL)getArgNotNull:(id)argValue commID:(NSString*)commandID{
    if (argValue == nil) {
        [self handleNullParams:commandID];
        return false;
    }
    return true;
}

-(void)handleNullParams:(NSString *) error_CallbackId {
    CDVPluginResult* result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString: CODE_INVALID_PARAMS];
    [self.commandDelegate sendPluginResult:result callbackId: error_CallbackId];
}

/*
    - post method
*/
-(void)act_mtls_post:(CDVInvokedUrlCommand *)command {
    /*
     if ok, get params with right order (define in js)
    */
    NSString *hostName = [command argumentAtIndex:0];
    NSString *endpt = [command argumentAtIndex:1];
    NSString *contentType = [command argumentAtIndex:2];
    NSString *dataString = [command argumentAtIndex:3];
    NSString *authenValue = [command argumentAtIndex:4];
    
    /*
     check Notnull for params
     */
    NSString *cm_id = command.callbackId;
    if (![self getArgNotNull:hostName commID:cm_id]) {
        return;
    }
    if (![self getArgNotNull:endpt commID:cm_id]) {
        return;
    }
    if (![self getArgNotNull:contentType commID:cm_id]) {
        return;
    }
    if (![self getArgNotNull:dataString commID:cm_id]) {
        return;
    }
    if (![self getArgNotNull:[command argumentAtIndex:5] commID:cm_id]) {
        return;
    }
    int timeoutInMilliseconds = [[command argumentAtIndex:5] intValue];
    
    /*
        -create config with timeout
        -create session with config
        -create request with httpmethod, body, url, contentType, authen
    */
    NSURLSessionConfiguration *config = [NSURLSessionConfiguration defaultSessionConfiguration];
    config.timeoutIntervalForRequest = timeoutInMilliseconds/1000;
    
    NSString *urlStr = [hostName stringByAppendingString:endpt];
    NSURL *url = [NSURL URLWithString:urlStr];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:config];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    request.HTTPMethod = @"POST";
    [request setValue:contentType forHTTPHeaderField:@"Content-Type"];
    [request setValue:authenValue forHTTPHeaderField:@"Authorization"];
    request.HTTPBody = [dataString dataUsingEncoding:NSUTF8StringEncoding];
    /*
        - handle in background
        - start download by NSURLSessionDataTask(reference ios development guide) with session & request
        - return result response (OK or FAIL)
    */
    [self.commandDelegate runInBackground:^{
        NSURLSessionDataTask *task = [session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
            NSString* callbackId = command.callbackId;
            CDVPluginResult *result = [self handleRespone:data res:response error:error];
            [self.commandDelegate sendPluginResult:result callbackId:callbackId];
        }];
        [task resume];
    }];
}

/*
 - get method
*/

-(void)act_mtls_get:(CDVInvokedUrlCommand *)command {
    /*
     if ok, get params with right order (define in js)
     */
    NSString *hostName = [command argumentAtIndex:0];
    NSString *endpt = [command argumentAtIndex:1];
    NSString *authenValue = [command argumentAtIndex:2];
    
    /*
     check Notnull for params
     */
    NSString *cm_id = command.callbackId;
    if (![self getArgNotNull:hostName commID:cm_id]) {
        return;
    }
    if (![self getArgNotNull:endpt commID:cm_id]) {
        return;
    }
    if (![self getArgNotNull:[command argumentAtIndex:3] commID:cm_id]) {
        return;
    }

    int timeoutInMilliseconds = [[command argumentAtIndex:3]intValue];
    
    /*
     -create config with timeout
     -create session with config
     -create request with httpmethod, url, authen
     */
    NSURLSessionConfiguration *config = [NSURLSessionConfiguration defaultSessionConfiguration];
    config.timeoutIntervalForRequest = timeoutInMilliseconds / 1000;
    
    NSString *urlStr = [hostName stringByAppendingString:endpt];
    NSURL *url = [NSURL URLWithString:urlStr];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:config];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    request.HTTPMethod = @"GET";
    [request setValue:authenValue forHTTPHeaderField:@"Authorization"];
    
    /*
     - handle in background
     - start download by NSURLSessionDataTask(reference ios development guide) with session & request
     - return result response (OK or FAIL)
     */
    [self.commandDelegate runInBackground:^{
        NSURLSessionDataTask *task = [session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
            NSString* callbackId = command.callbackId;
            CDVPluginResult *result = [self handleRespone:data res:response error:error];
            [self.commandDelegate sendPluginResult:result callbackId:callbackId];
        }];
        [task resume];
    }];
}

    
 /*
    - return CDVPluginResult with response , data, error
 */
-(CDVPluginResult *)handleRespone:(NSData *) data res:(NSURLResponse *)response error:(NSError *)error {
    CDVPluginResult *result;
    /*
     - if no error, send result with statusCode of response
        + 200 <= statusCode < 300 --> result_OK with content respone (responeCode, header, respone)
        + >= 400 --> result_ERROR with statusCode
     - if error, --> result_ERROR with error
    */
    if (error == nil) {
        if ([response isKindOfClass:[NSHTTPURLResponse class]]) {
            NSInteger statusCode = [(NSHTTPURLResponse *)response statusCode];
            NSLog(@"dataTaskWithRequest HTTP status code: %li", (long)statusCode);
            if (statusCode >= 200 && statusCode < 300) {
                NSDictionary* headers = [(NSHTTPURLResponse *)response allHeaderFields];
                NSDictionary *jsonRes = [NSJSONSerialization JSONObjectWithData:data options:kNilOptions error:&error];
                NSDictionary *dict = @{ @"responseCode" : [NSNumber numberWithInteger:statusCode], @"response" : jsonRes, @"header": headers};
                result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:dict];
            } else if(statusCode >= 400) {
                result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsNSInteger:statusCode];
            }
        }
    } else {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error.description];
    }
    return result;
}

@end
