//
//  VGuardPlugin.m
//
//
//  MIT licensed
//

#import "VGuardPlugin.h"
#import <VGuard/VGuard.h>

#pragma mark -

@interface VGuardPlugin () <VGuardManagerProtocol, VGuardExceptionHandlerProtocol>
{
    VGuardManager *vGuardMgr;
}
@property (nonatomic, copy, readwrite) NSString *scanCallbackId;
@property (nonatomic, copy, readwrite) NSString *callbackId;
@end

#pragma mark -

@implementation VGuardPlugin

- (void)initalize:(CDVInvokedUrlCommand *)command{
    self.callbackId = command.callbackId;
    // setup VGuardExceptionHandler before V-Guard Instance
    VGuardExceptionHandler *exceptionHandler = [VGuardExceptionHandler sharedManager];
    [exceptionHandler setDelegate:self];
        
    // V-Guard Instance
    vGuardMgr = [VGuardManager sharedVGuardManager];
    [vGuardMgr setDelegate:self];
    [vGuardMgr setIsDebug:YES];

    [self.commandDelegate runInBackground:^{
        [vGuardMgr initializeVGuard];

        [vGuardMgr start];
    }];
}

#pragma Mark - VGuard Delegate
- (void)statusVOS:(VOS_STATUS)status withError:(NSError *)error
{
    NSLog(@"statusVOS %d -- %@",status,error);
    if (status==VOS_OK) {
//        NSDate *methodStart = [NSDate date];
//        unsigned char password[] = { '6', '6', '6', '6', '6', '6' };
//        NSData* pwdData = [NSData dataWithBytes:password length:sizeof(password)];
//        [VOSSecureFileIO initSecureFileIO: pwdData];
        // sqlite3_enableEncryption(true); // To Enable Secure SQLite Database
//
//        NSDate *methodFinish = [NSDate date];
//        NSTimeInterval executionTime = [methodFinish timeIntervalSinceDate:methodStart];
//        NSLog(@"secure_file_io initalization= %f", executionTime);
    }
}

- (void)statusVGuard:(VGUARD_STATUS)status withError:(NSError *)error
{
    NSLog(@"statusVGuard %d -- %@",status,error);
    if(!error){
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                             messageAsInt:status];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:self.callbackId];

    }else{
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                             messageAsInt:status];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:self.callbackId];
    }
}

- (void)sslErroDetected:(NSError *)error{
    NSLog(@"sslErroDetected %@",error);
}

- (void)vGuardDidFinishInitializing:(BOOL)status withError:(NSError *)error {
    NSLog(@"vGuardDidFinishInitializing: %@", status ? @"SUCCESS" : @"FAILED");
    if (status) {
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:@"success"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:self.callbackId];
    }
    else
    {
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                          messageAsString:@"failed"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:self.callbackId];
    }
}

#pragma mark - VGuardExceptionHandlerProtocol
- (void)vGuardExceptionHandler:(NSException *)exception {
    NSLog(@"vGuardExceptionHandler: %@", exception);
}

#pragma Mark - Public Methods
- (void)act_setup_vguard:(CDVInvokedUrlCommand *)command{
    [self initalize:command];
}

- (void)act_start_vguard:(CDVInvokedUrlCommand *)command{
    self.callbackId = command.callbackId;
    [self.commandDelegate runInBackground:^{
        [vGuardMgr start];
    }];
}

//modified by An
- (void)act_resume_vguard:(CDVInvokedUrlCommand *)command{
    self.callbackId = command.callbackId;
    [self.commandDelegate runInBackground:^{
        [vGuardMgr start];
    }];
}

- (void)act_pause_vguard:(CDVInvokedUrlCommand *)command{
    //?????
}

- (void)act_version_vguard:(CDVInvokedUrlCommand *)command{
    [self.commandDelegate runInBackground:^{
        NSString *version = [vGuardMgr getVGuardVersionInformation];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:version];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

#pragma Mark - Secure Keyboard
- (void)act_enable_Keyboard:(CDVInvokedUrlCommand *)command{
    [self isMissingParam:1 command:command];
    BOOL enable = [[[command arguments] objectAtIndex:0]boolValue];
    VKeySecureKeypad *kb = [VKeySecureKeypad sharedModule];
    [kb setEnableKeyboard:enable];
    
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                        messageAsBool:YES];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}
    
- (void)act_enable_ScrambleKeypad:(CDVInvokedUrlCommand *)command{
    [self isMissingParam:1 command:command];
    BOOL enable = [[[command arguments] objectAtIndex:0]boolValue];
    VKeySecureKeypad *kb = [VKeySecureKeypad sharedModule];
    [kb setEnableScrambleKeypad:enable];
    
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                        messageAsBool:YES];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}
    
- (void)act_set_keypad_button_alpha:(CDVInvokedUrlCommand *)command{
    [self isMissingParam:1 command:command];
    float alpha = [[[command arguments] objectAtIndex:0]floatValue];
    VKeySecureKeypad *kb = [VKeySecureKeypad sharedModule];
    [kb setKeypadButtonAlpha:alpha];
    
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                        messageAsBool:YES];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}
    
- (void)act_set_keypad_button_background_color:(CDVInvokedUrlCommand *)command{
    [self isMissingParam:1 command:command];
    NSString* color = [[command arguments] objectAtIndex:0];
    VKeySecureKeypad *kb = [VKeySecureKeypad sharedModule];
    [kb setKeyboardButtonBackgroundColor:[self colorFromHexString:color]];
    
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                        messageAsBool:YES];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}
    
- (void)act_set_keypad_text_color:(CDVInvokedUrlCommand *)command{
    [self isMissingParam:1 command:command];
    NSString* color = [[command arguments] objectAtIndex:0];
    VKeySecureKeypad *kb = [VKeySecureKeypad sharedModule];
    [kb setKeypadTextColor:[self colorFromHexString:color]];
    
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                        messageAsBool:YES];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}
    
- (UIColor *)colorFromHexString:(NSString *)hexString {
    unsigned rgbValue = 0;
    NSScanner *scanner = [NSScanner scannerWithString:hexString];
    [scanner setScanLocation:1]; // bypass '#' character
    [scanner scanHexInt:&rgbValue];
    return [UIColor colorWithRed:((rgbValue & 0xFF0000) >> 16)/255.0 green:((rgbValue & 0xFF00) >> 8)/255.0 blue:(rgbValue & 0xFF)/255.0 alpha:1.0];
}
    
-(BOOL)isMissingParam:(int)num command:(CDVInvokedUrlCommand *)command{
    if (command.arguments.count < num){
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                          messageAsString:@"MISSING PARAMS"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
        return YES;
    };
    return NO;
}

@end

