/**
 * @file Reuse the VGuard's SecureHttpURLConnection.
 */

(function() {
	var root, SecureHTTPConnectionPlugin;

	root = this;
	
	SecureHTTPConnectionPlugin = function() {}; 
		/**
		 * Constants
		 */
	var CONST = {
		PLG_NAME : "SecureHTTPConnectionPlugin",
		ACT_ALERT_TEST: "act_mtls_test",
		ACT_MTLS_GET: "act_mtls_get",
		ACT_MTLS_POST: "act_mtls_post",
	};

	/**
	 * Shows an alert using provided args.
	 * @param {Function} successCallback success
	 * @param {Function} successCallback error
	 * @param {Object} args
	 */
	SecureHTTPConnectionPlugin.prototype.test = function(success, error, args) {
		cordova.exec(success, error, CONST.PLG_NAME, CONST.ACT_ALERT_TEST, args);
	};
			
	// MTLS Functions

	/**
	 * GET
	 * @param {string} hostUrl
	 * @param {string} endPoint
	 * @param {string} reqAuthHeader; null if there is no authentication
	 * @param {string} timeoutInMilliseconds; timeoutInMilliseconds, default is 15000
	 * @param {object} onSuccess; success callback function
	 * @param {object} onError; error callback function
	 */
	function onGetURL(hostUrl, endPoint, reqAuthHeader, onSuccess, onError, timeoutInMilliseconds=15000) {
        if (isEmpty(hostUrl) || isEmpty(endPoint)) {
            onError("params should be not empty");
            return;
        }
		cordova.exec(
			// successcb
			function (response) {
				if(response) {
					//console.log("mtls_GET: successcb");
					if(onSuccess) {
						onSuccess(response);
					}
				}
			},
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					//console.log("mtls_GET errorcb");
					onError(errorcbmsg);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_MTLS_GET,
			[hostUrl, endPoint, reqAuthHeader, timeoutInMilliseconds]
		);
	};
	

	/**
	 * POST
	* @param {string} hostUrl
	* @param {string} endPoint
	* @param {string} contentType
	* @param {string} dataAsString
	* @param {string} reqAuthHeader; null if there is no authentication
	* @param {string} timeoutInMilliseconds; timeoutInMilliseconds, default is 15000
	*/
	function onPostURL(hostUrl, endPoint, contentType, dataAsString, reqAuthHeader, onSuccess, onError, timeoutInMilliseconds=15000) {
        if (isEmpty(hostUrl) || isEmpty(endPoint) || isEmpty(contentType) || isEmpty(dataAsString)) {
            onError("params should be not empty");
            return;
        }
        cordova.exec(
            // successcb
            function (response) {
                if(response) {
                    //console.log("mtls_POST: successcb");
                    if(onSuccess) {
                        onSuccess(response);
                    }
                }
            },
            // errorcb
            function (errorcbmsg) {
                if(errorcbmsg) {
                    //console.log("mtls_POST errorcb");
                    onError(errorcbmsg);
                }
            },
            CONST.PLG_NAME,
            CONST.ACT_MTLS_POST,
            [hostUrl, endPoint, contentType, dataAsString, reqAuthHeader, timeoutInMilliseconds]
        );
    };

	function isEmpty(str)
    {
        if (typeof str == 'undefined' || !str || str.length === 0 || str === "" || !/[^\s]/.test(str) || /^\s*$/.test(str) || str.replace(/\s/g,"") === "")
        {
            return true;
        }
        else
        {
            return false;
        }
    }

	/*
		* VGSecureHTTPConnFactory
		*/
	VGSecureHTTPConnFactory = {
		test: function(title, message, buttonLabel) {
			cordova.exec(null, null, "SecureHTTPConnectionPlugin", "act_alert_test", [title, message, buttonLabel]);
		},
		onGetURL:function(hostUrl, endPoint, reqAuthHeader, onSuccess, onError, timeoutInMilliseconds){
			onGetURL(hostUrl, endPoint, reqAuthHeader, onSuccess, onError, timeoutInMilliseconds);
		},
		onPostURL:function(hostUrl, endPoint, contentType, dataAsString, reqAuthHeader, onSuccess, onError, timeoutInMilliseconds){
			onPostURL(hostUrl, endPoint, contentType, dataAsString, reqAuthHeader, onSuccess, onError, timeoutInMilliseconds);
		}

	};
	
	root.secureHTTPConnectionPlugin = {
			test: VGSecureHTTPConnFactory.test,
			onGetURL: VGSecureHTTPConnFactory.onGetURL,
			onPostURL: VGSecureHTTPConnFactory.onPostURL,
	};
	
}).call(this);
