/*
 * (C) Copyright 2016 V-Key Solutions.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 *     	Damith Hettige
 * 		Rukmal Dias
 */

(function() {
	var root, VGuardSpPlugin;
  	root = this;
	  
	/**
     * Constructor
     */
  	VGuardSpPlugin = function() {
  	};
	  	
  	/**
  	 * Constants
  	 */
  	var CONST = {
  		PLG_NAME : "VGuardSpPlugin",
  		ACT_SP_PUT : "act_sp_put",
  		ACT_SP_GET : "act_sp_get",
  		ACT_SP_REMOVE : "act_sp_remove",
  		ACT_SP_CONTAINS_KEY : "act_sp_contains_key",
  		ACT_SP_CLEAR : "act_sp_clear",
  	};
  	
  	/**
  	 * Saves key, value in secure preferences
	 * @param {Object} key
	 * @param {Object} value
	 * @param {Object} status
  	 */
  	VGuardSpPlugin.prototype.spput = function(successcb, errorcb, key, value) {
	  	cordova.exec(
	  		function (success) {
	  			if(successcb) {
	  				successcb(success);
	  			}
	  		}, 
	  		function (error) {
	  			if(errorcb) {
	  				errorcb(error);
	  			}
	  		},
	  		CONST.PLG_NAME, 
	  		CONST.ACT_SP_PUT,
	  		[key, value]);
	};
	
	/**
	 * Returns a stored value from secure preferences
	 * @param {Object} key
	 * @param {Object} status
	 */
	VGuardSpPlugin.prototype.spget = function(successcb, errorcb, key) {
		cordova.exec(
	  		function (success) {
	  			if(successcb) {
	  				successcb(success);
	  			}
	  		}, 
	  		function (error) {
	  			if(errorcb) {
	  				errorcb(error);
	  			}
	  		},
	  		CONST.PLG_NAME, 
	  		CONST.ACT_SP_GET,
	  		[key]);
	};
	
	/**
	 * Removes a key, value pair from secure preferences
	 * @param {Object} key
	 * @param {Object} status
	 */
	VGuardSpPlugin.prototype.spremove = function(successcb, errorcb, key) {
		cordova.exec(
	  		function (success) {
	  			if(successcb) {
	  				successcb(success);
	  			}
	  		}, 
	  		function (error) {
	  			if(errorcb) {
	  				errorcb(error);
	  			}
	  		}, 
	  		CONST.PLG_NAME, 
	  		CONST.ACT_SP_REMOVE,
	  		[key]);
	};
	
	/**
	 * Finds whether a particular key exists in secure preferences
	 * @param {Object} key
	 * @param {Object} status
	 */
	VGuardSpPlugin.prototype.spcontainskey = function(successcb, errorcb, key) {
		cordova.exec(
	  		function (success) {
	  			if(successcb) {
	  				successcb(success);
	  			}
	  		}, 
	  		function (error) {
	  			if(errorcb) {
	  				errorcb(error);
	  			}
	  		},
	  		CONST.PLG_NAME, 
	  		CONST.ACT_SP_CONTAINS_KEY,
	  		[key]);
	};
	
	/**
	 * Removes all key, value pairs from secure preferences
	 * @param {Object} status
	 */
	VGuardSpPlugin.prototype.spclear = function(successcb, errorcb) {
		cordova.exec(
	  		function (success) {
	  			if(successcb) {
	  				successcb(success);
	  			}
	  		}, 
	  		function (error) {
	  			if(errorcb) {
	  				errorcb(error);
	  			}
	  		},
	  		CONST.PLG_NAME, 
	  		CONST.ACT_SP_CLEAR,
	  		[]);
	};
			  	
	VGuardSpFactory = {
		openPreferences : function() {
			return new VGuardSpPlugin();
		}
	};
  	
	root.vguardspplugin = {
		openPreferences : VGuardSpFactory.openPreferences
	};
  
}).call(this);
