var argscheck = require('cordova/argscheck'),
	utils = require('cordova/utils'),
	exec = require('cordova/exec');
/*
 * (C) Copyright 2016 V-Key Solutions.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 *     	Damith Hettige
 * 		Rukmal Dias
 */

	  
	/**
     * Request a vguard instance to handle vguard operations.
     */
  	VGuardPlugin = function() {
  	};
	  	
  	/**
  	 * Constants
  	 */
  	var CONST = {
  		PLG_NAME : "VGuardPlugin",
  		ACT_SETUP_VGUARD : "act_setup_vguard",
        ACT_SEND_EMAIL: "act_send_email",
  		ACT_VGUARD_PAUSE : "act_vguard_pause",
  		ACT_VGUARD_RESUME : "act_vguard_resume",
  		ACT_ALERT_TEST : "act_alert_test",
  		CB_SCAN_COMPLETE : "cb_scan_complete",
  		CB_VOS_READY : "cb_vos_ready",
  		CB_VGUARD_INITIALIZE : "cb_vguard_initialize",
  		CB_VGUARD_ERROR : "cb_vguard_error",
  	};
  
	/**
	 * Shows an alert using provided args.
	 * @param {Function} successCallback success
	 * @param {Function} successCallback error
	 * @param {Object} args
	 */
	VGuardPlugin.prototype.test = function(success, error, args) {
	  	cordova.exec(success, error, CONST.PLG_NAME, CONST.ACT_ALERT_TEST, args);
	};
	
	/**
	 * Starts VGuard protection service
 	 * @param {Function} onProfileLoad 
 	 * @param {Function} onVosReady
 	 * @param {Function} onScanComplete
 	 * @param {Function} onError
	 */	
	VGuardPlugin.prototype.setupVGuard = function (onVGuardInitialize, onVosReady, onScanComplete, onError) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
		  			var msg = successcbmsg;
		  			console.log("setupVGuard: successcb");
							  			
		  			if(msg && msg.action == CONST.CB_VGUARD_INITIALIZE) {
		  				console.log("-- on vguard initialize");
		  				if(onVGuardInitialize) {
		  					onVGuardInitialize();
		  				}
		  			}
		  			else if(msg && msg.action == CONST.CB_VOS_READY) {
		  				console.log("-- on vos ready");
		  				if(onVosReady) {
		  					onVosReady(msg.values);
		  				}
		  			}
		  			else if(msg && msg.action == CONST.CB_SCAN_COMPLETE) {
		  				console.log("-- on scan complete");
		  				if(onScanComplete) {
		  					onScanComplete(msg.values);
		  				}
		  			}
		  			else if(msg && msg.action == CONST.CB_VGUARD_ERROR) {
		  				// should not come here..
		  				console.warn("-- on vguard error");
		  			}
		  		}
			}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
		  			console.log("setupVGuard errorcb : " + errorcbmsg.values);
										
					if(msg && msg.action == CONST.CB_VGUARD_ERROR) {
		  				console.warn("-- on vguard error");
		  				if(onError) {
		  					onError(msg.values);
		  				}
		  			}
				}
			}, 
			CONST.PLG_NAME, 
			CONST.ACT_SETUP_VGUARD,
			[]
		);
    };

    VGuardPlugin.prototype.sendEmail = function (onSuccess, onError, email) {
        cordova.exec(
            function(successcbmsg){
                if(successcbmsg){
                    var msg = errorcbmsg;
                    console.log("call send email: successcb");
                    onSuccess(msg)
                }
			},
			function (errorcbmsg){
				if (errorcbmsg) {
				    var msg = errorcbmsg;
					console.log("call send email: failed");
					onError(msg)
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_SEND_EMAIL,
			[email]
        );
	};
	
	
	VGuardPlugin.prototype.onAppPause = function(onSuccess, onError) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
		  			console.log("onAppPause: successcb");
		  			if(onSuccess) {
		  				onSuccess();
		  			}
		  		}
		  	}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
		  			console.log("onAppPause errorcb : " + errorcbmsg.values);
										
					if(msg && msg.action == CONST.CB_VGUARD_ERROR) {
		  				console.warn("-- on vguard error");
		  				if(onError) {
		  					onError(msg.values);
		  				}
		  			}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_VGUARD_PAUSE,
			[]
		);
	};
	
	VGuardPlugin.prototype.onAppResume = function(onSuccess, onError) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
		  			console.log("onAppResume: successcb");
		  			if(onSuccess) {
		  				onSuccess();
		  			}
		  		}
		  	}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
		  			console.log("onAppResume errorcb : " + errorcbmsg.values);
										
					if(msg && msg.action == CONST.CB_VGUARD_ERROR) {
		  				console.warn("-- on vguard error");
		  				if(onError) {
		  					onError(msg.values);
		  				}
		  			}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_VGUARD_RESUME,
			[]
		);	
	};

module.exports = new VGuardPlugin();

